import math
import matplotlib as mpl
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
from tqdm import tqdm
import numpy as np
import pandas as pd
import os
import csv
# import pyqtgraph as pg
import pickle
import PurpleMicroStar.BasicConst as BC
import PurpleMicroStar.BasicMech as BMech
import PurpleMicroStar.StarBody as Sbody
from prettytable import PrettyTable

'''
### This module is built for Multibody motion simulation

@AUTHOR: Yufeng Wang
@DATE: 2022-6
@LICENSE: GPL-V3 license

CopyRight (C) Yufeng-Wang/Airscker, 2022-6

More information about this module please see: https://airscker.github.io/Purple-Micro-Star/
'''

class MultiBody:
    '''
    ## Buid a cluster of planets, allow they evolution via time and store their Astrophysics properties
    ### Parameter:
        manybody: a list of Starbody Objects

    ### Optional operation:
        You can build your new cluster via Dataset(), more information please refer to function Dataset()

        And you can rebuild your former cluster stored in some files, more information please refer to function Rebuild()
    '''
    def __init__(self,manybody=[]):
        self.mbody=manybody
        self.data=None
        self.pos_data=None
        self.vel_data=None
        self.Crashed=[]
    def __str__(self):
        str1='\n---Basic Information of Multibody---\n'
        str2='\n{}'.format(self.data)
        # str3=''
        return str1+str2
    def Iteration(self,dt=0.001,iterCount=100000,output='.\\Curve_data'):
        '''
        ## Calculate all planets' motion properties, as well as their body properties, then save them into files
        ### Parameters:
        - dt: the time period per motion used
        - iterCount: the iteration times of dt

        ### Return:
        - Pos_curve: The position information of every planet in every iterated time, in the form of nparray\n
                [[[p1_t1],[p2_t1],[p3_t1]],
                 [[p1_t2],[p2_t2],[p3_t2]],...],shape=(iter,body_num,3)
        - Vel_curve: The velocity information of every planet in every iterated time, in the form of nparray\n
                [[[v1_t1],[v2_t1],[v3_t1]],
                 [[v1_t2],[v2_t2],[v3_t2]],...]
        '''
        print('\n---Total time of motion: {}s, total iteration times: {}---\n'.format(iterCount*dt,iterCount))
        Pos_curve=[]
        Vel_curve=[]
        bar=tqdm(range(iterCount),mininterval=1)
        mbody=self.mbody.copy()
        for iter in bar:
            bar.set_description('Calculating Multibody Motion Properties at {}s'.format(round(iter*dt,5)))
            # Calculate their gravity force and roche_limit
            for i in range(len(mbody)):
                mcopy=mbody.copy()
                mcopy.pop(i)
                mbody[i].Force(mcopy)
                mbody[i].Roche_limit(mcopy)
                
            # Delete Crashed Starbody
            for j in range(len(mbody)):
                for i in range(len(mbody)):
                    if mbody[i].ad_prop['Roche_limit']/mbody[i].mass>=1:
                        self.Crashed.append(mbody[i])
                        mbody.pop(i)
                        break

            # Iterate their motion information
            for i in range(len(mbody)):
                mbody[i].Iteration(dt=dt,iterCount=1)

            # Store body data
            for i in range(len(mbody)):
                self.mbody[self.mbody.index(mbody[i])]=mbody[i]

            # Get pos/vel curve data
            Pos_curve.append([0]*len(self.mbody))
            Vel_curve.append([0]*len(self.mbody))
            for i in range(len(self.mbody)):
                Pos_curve[iter][i]=self.mbody[i].pos
                Vel_curve[iter][i]=self.mbody[i].vel
        
        # Prepare for data saving
        Pos_curve=np.array(Pos_curve)
        Vel_curve=np.array(Vel_curve)
        # Save curve data
        try:
            os.makedirs(output)
        except:
            pass
        # Save body basic information
        with open('{}\\Multibody_{}_Iter_{}_dt_{}_body_Rebuild.csv'.format(output,len(self.mbody),iterCount,dt),'w',newline='')as f:
            csv_writer=csv.writer(f)
            csv_writer.writerow(['Name','Mass','Radius','sx','sy','sz'])
            for i in range(len(self.mbody)):
                csv_writer.writerow([self.mbody[i].name,self.mbody[i].mass,self.mbody[i].Radius,self.mbody[i].spin[0],self.mbody[i].spin[1],self.mbody[i].spin[2]])
        f.close()
        # # Save iteration image
        # plt.imsave('{}\\Multibody_{}_Iter_{}_Position.jpg'.format(output,len(self.mbody),iterCount),pos_norm)
        # plt.imsave('{}\\Multibody_{}_Iter_{}_Velocity.jpg'.format(output,len(self.mbody),iterCount),vel_norm)
        # Save position and velocity data
        with open('{}\\Multibody_{}_Iter_{}_dt_{}_Position.pkl'.format(output,len(self.mbody),iterCount,dt),'wb')as f:
            pickle.dump(Pos_curve,f)
        f.close()
        with open('{}\\Multibody_{}_Iter_{}_dt_{}_Velocity.pkl'.format(output,len(self.mbody),iterCount,dt),'wb')as f:
            pickle.dump(Vel_curve,f)
        f.close()

        print('\n---Iteration over, all data saved---\n')
        self.pos_data=Pos_curve
        self.vel_data=Vel_curve

        
        if self.Crashed!=[]:
            print('\n---StarBody Crashed are listed here---\n')
            str1=PrettyTable()
            str1.field_names=['StarBody Name','Roche Limit','Iteration Times']
            for i in range(len(self.Crashed)):
                str1.add_row([self.Crashed[i].name,self.Crashed[i].ad_prop['Roche_limit'],self.Crashed[i].iter])
            print(str1)
        return Pos_curve,Vel_curve
    def Dataset(self,datapath='.\\exampledata\\Multibody.csv',datasep=','):
        '''
        ## Load MultiBody From Given formated Dataset .csv file
        ### Parameters:
        - datapath: the csv data root path of planets' basic data
        - datasep: the seperation of every information

        ### Columns' name:  
            Name,x,y,z,vx,vy,vz,Mass,Radius,Spinx,Spiny,Spinz\n
            If you don't obey this rule, the result may boom

        ### Return:
            The list of Object 'Starbody'
        '''
        # read data
        try:
            self.data=pd.read_csv(datapath,sep=datasep)
        except:
            print('Check the data file path!')
            return None,None
        body=[]
        # create starbody objects
        try:
            for i in range(self.data.shape[0]):
                body.append(Sbody.StarBody(Position=[self.data.iloc[i][1],self.data.iloc[i][2],self.data.iloc[i][3]],
                                    Velocity=[self.data.iloc[i][4],self.data.iloc[i][5],self.data.iloc[i][6]],
                                    Mass=self.data.iloc[i][7],
                                    Radius=self.data.iloc[i][8],
                                    Spin=[self.data.iloc[i][9],self.data.iloc[i][10],self.data.iloc[i][11]],
                                    Name=self.data.iloc[i][0]))
            self.mbody=body.copy()
        except:
            print('Check the sep or column format of data!')
            return None,None
        # Initailize their basic information
        for i in range(len(self.mbody)):
            mcopy=self.mbody.copy()
            mcopy.pop(i)
            self.mbody[i].Force(mcopy)
            self.mbody[i].Roche_limit(mcopy)
        return self.mbody,self.data
    def Rebuild(self,body='.\\exampledata\\Curve_data\\Multibody_3_Iter_100_dt_10_body_Rebuild.csv',
                position='.\\exampledata\\Curve_data\\Multibody_3_Iter_100_dt_10_Position.pkl',
                velocity='.\\exampledata\\Curve_data\\Multibody_3_Iter_100_dt_10_Velocity.pkl'):
        '''
        ## Rebuild your planet cluster using data files saved by Iteration(), make sure you haven't changed anything of these data, you just need to find the cooresponding filenames
        ### Parameters:
        - body: the $body_Rebuild.csv path
        - position: the $Position.csv path
        - Velocity: the $Velocity.csv path
        ### Return:
        - The list of position curve data, The list of the velocity curve data, The list of Object 
        '''
        # Get basic body info
        body_data=pd.read_csv(body)
        body_name=body_data['Name']
        body_mass=body_data['Mass']
        body_radius=body_data['Radius']
        body_spinx=body_data['sx']
        body_spiny=body_data['sy']
        body_spinz=body_data['sz']
        # Get position and velocity data
        with open(position,'rb')as f:
            pos_data=pickle.load(f)
        f.close()
        with open(velocity,'rb')as f:
            vel_data=pickle.load(f)
        f.close()

        # Rebuild Starbody Object cluster
        manybody=[]
        for i in range(len(body_name)):
            manybody.append(Sbody.StarBody(Position=pos_data[-1][i],Velocity=vel_data[-1][i],Mass=body_mass[i],Radius=body_radius[i],Spin=[body_spinx[i],body_spiny[i],body_spinz[i]],Name=body_name[i]))
        for i in range(len(manybody)):
            m_copy=manybody.copy()
            m_copy.pop(i)
            manybody[i].Force(m_copy)
        self.mbody=manybody
        self.pos_data=pos_data
        self.vel_data=vel_data
        return pos_data,vel_data,manybody
    def Plot(self):
        '''
        ## Plot the motion curve of Multibodies
        '''
        print('---Plotting motion curve of Multibodies---')
        body_x=[]
        body_y=[]
        body_z=[]
        try:
            for i in range(self.pos_data.shape[1]):
                body_x.append([])
                body_y.append([])
                body_z.append([])
                for j in range(self.pos_data.shape[0]):
                    body_x[i].append(self.pos_data[j][i][0])
                    body_y[i].append(self.pos_data[j][i][1])
                    body_z[i].append(self.pos_data[j][i][2])
        except:
            print('---Iteration need to done before Plotting---')
            return None
        body_x=np.array(body_x)
        body_y=np.array(body_y)
        body_z=np.array(body_z)
        # Plot static image
        figure=plt.figure()
        ax=Axes3D(figure,auto_add_to_figure=False)
        figure.add_axes(ax)
        ax.set_title('Motion curve of Multibodies')
        ax.set_xlabel("x")
        ax.set_ylabel("y")
        ax.set_zlabel("z")
        # figure.colorbar(mpl.cm.ScalarMappable())
        for i in range(len(body_x)):
            ax.plot3D(body_x[i],body_y[i],body_z[i],label=self.mbody[i].name)
        for i in range(self.pos_data[-1].shape[0]):
            ax.scatter3D(self.pos_data[-1][i][0],self.pos_data[-1][i][1],self.pos_data[-1][i][2],s=100*self.mbody[i].Radius/BC.R_earth)
        ax.legend()
        plt.show()
        return [body_x,body_y,body_z]

            


    



'''
# Gravitational Wave Simulator
Every massive object that accelerates produces gravitational waves.This includes humans, cars, airplanes etc., 
but the masses and accelerations of objects on Earth are far too small to make gravitational waves big enough to detect with our instruments.
To find big enough gravitational waves, we have to look far outside of our own solar system.

It turns out that the Universe is filled with incredibly massive objects that undergo rapid accelerations that by their nature,
generate gravitational waves that we can actually detect. Examples of such things are orbiting pairs of black holes and neutron stars,
or massive stars blowing up at the ends of their lives.

LIGO scientists have defined four categories of gravitational waves based on what generates them:
- Continuous
- Compact Binary Inspiral
- Stochastic
- Burst

Each category of objects generates a unique or characteristic set of signal that interferometers can sense,
but not all types of Gravitational Wave can be simulated numerically. This module simulate the gravitational wave of *Compact Binary Inspiral Gravitational Wave* and *Continuous Gravitational Wave*

More information about this please see: https://www.ligo.caltech.edu/page/gw-sources

@AUTHOR: Yufeng Wang
@DATE: 2022-6
@LICENSE: GPL-V3 license

CopyRight (C) Yufeng-Wang/Airscker, 2022-6

More information about this module please see: https://airscker.github.io/Purple-Micro-Star/
'''
import pickle as pkl
import PurpleMicroStar.MultiBody as MB
import PurpleMicroStar.StarBody as SB
import PurpleMicroStar.BasicConst as BC
import PurpleMicroStar.BasicMech as BMech
import numpy as np
import math
import prettytable as PT
import matplotlib.pyplot as plt
import os
from tqdm import tqdm


N=32/5

class CBIGW:
    '''
    # Compact Binary Inspiral Gravitational Waves\n
    Compact binary inspiral gravitational waves are produced by orbiting pairs of massive and dense ("compact") objects like white dwarf stars, black holes, and neutron stars.\n
    There are three subclasses of "compact binary" systems in this category of gravitational-wave generators:
    - Binary Neutron Star (BNS)
    - Binary Black Hole (BBH)
    - Neutron Star-Black Hole Binary (NSBH)
    
    Each binary pair creates a unique pattern of gravitational waves, but the mechanism of wave-generation is the same across all three. It is called "inspiral".

    Inspiral occurs over millions of years as pairs of dense compact objects revolve around each other.
    As they orbit, they emit gravitational waves that carry away some of the system's orbital energy.
    As a result, over eons, the objects orbit closer and closer together.
    Unfortunately, moving closer causes them to orbit each other faster, which causes them to emit stronger gravitational waves,
    which causes them to lose more orbital energy, inch ever closer, orbit faster, lose more energy, move closer, orbit faster... etc.
    The objects are doomed, inescapably locked in a runaway accelerating spiraling embrace.

    ### Parameters:\n
    - m1,m2: The mass of planets\n
    - r1,r2: The orbit radius of per planets\n
    - Name: the name of binary planets\n
    '''
    def __init__(self,m1=2e2*BC.M_sun,m2=1e3*BC.M_sun,r1=1e-5*BC.AU,r2=2e-5*BC.AU,Name='CBIG_SYS1'):
        self.name=Name
        self.M=np.array([m1,m2])
        self.r_int=np.array([r1,r2])
        self.r=np.array([r1,r2])
        self.spin=Orbit_spin(self.M[0],self.M[1],np.sum(self.r_int))
        self.info1={'Name of Black Holes':['BlackHole1','BlackHole2'],'Mass of Black Holes':[m1,m2],'Initial Orbit Radius':[r1,r2],}
        self.Coalescence_time=np.sum(self.M)*np.sum(self.r_int)**4/(N*Reduce_mass(self.M[0],self.M[1])*Schwarz_Radius(np.sum(self.M))**3*BC.c)
        self.Cu_time=0
    def __str__(self):
        str1='\n---Basic information of Binary Black Hole System---\n\n'
        str2=PT.PrettyTable()
        str2.field_names=['Name of Planets','Mass of Planets(M_Sun)','Initial Orbit Radius(AU)']
        str2.add_rows([['Black Hole 1',self.M[0]/BC.M_sun,self.r_int[0]/BC.AU],['Black Hole 2',self.M[1]/BC.M_sun,self.r_int[1]/BC.AU]])
        print(str1,str2,'\n')
        str3='Total coalescence time: {}s\nCurrent time of the system: {}s\n'.format(self.Coalescence_time,self.Cu_time)
        str4='\n---More Properties are waiting for your exploration!---\n'
        return str3+str4
    def Grav_Wave(self,Obser_R=[1e3,1e3,1e3],T_now=200):
        '''
        ## Get the gravitational wave field
        ### Parameters:
        - Obser_R: the position vector of the observer
        - T_now: the time at the place of observation
        
        ### Return:
            The Gravitational Wave Field
        '''
        if T_now>self.Coalescence_time:
            print('---Coalescence Time {}s Reached!, Current Time: {}s---'.format(self.Coalescence_time,T_now))
            return 0
        # Basic args
        cu_r=self.Distance(T_now=T_now)
        mu=Reduce_mass(self.M[0],self.M[1])
        spin=Orbit_spin(self.M[0],self.M[1],cu_r)
        theta=np.deg2rad(BMech.Vec_Angle(Obser_R,[0,0,1]))
        t_r=T_now-BMech.Vec_Length(Obser_R)/BC.c
        R_len=BMech.Vec_Length(Obser_R)
        # Calculation
        coef=BC.G*mu*spin**3*cu_r**2/(2*BC.c**3*R_len)
        # field_x=np.sin(2*theta)*np.sin(2*spin*t_r)*np.cos(theta)*np.cos(spin*t_r)+2*np.sin(theta)*np.cos(2*spin*t_r)*np.sin(spin*t_r)
        # fiels_y=np.sin(2*theta)*np.sin(2*spin*t_r)*np.cos(theta)*np.sin(spin*t_r)-2*np.sin(theta)*np.cos(2*spin*t_r)*np.cos(spin*t_r)
        # field_z=-np.sin(theta)
        field_x=np.sin(2*theta)*np.sin(2*spin*t_r)*np.cos(theta)+2*np.sin(theta)*np.cos(2*spin*t_r)
        fiels_y=np.sin(2*theta)*np.sin(2*spin*t_r)*np.cos(theta)-2*np.sin(theta)*np.cos(2*spin*t_r)
        field_z=-np.sin(theta)
        field=np.array([field_x,fiels_y,field_z])
        # Set properties
        self.Cu_time=T_now
        self.spin=spin
        return coef*field
    def Distance(self,T_now=200):
        '''
        ## Get the distance of balckholes at specified time
        '''
        if T_now>self.Coalescence_time:
            print('---Coalescence Time {}s Reached!, Current Time: {}s---'.format(self.Coalescence_time,T_now))
            return 0
        cu_r=math.pow((np.sum(self.r_int)**4-(N*(Reduce_mass(self.M[0],self.M[1])/np.sum(self.M))*BC.c*T_now*Schwarz_Radius(np.sum(self.M))**3)),1/4)
        self.r[0]=cu_r*self.M[1]/(np.sum(self.M))
        self.r[1]=cu_r*self.M[0]/(np.sum(self.M))
        self.Cu_time=T_now
        return cu_r
    def DEnergy_Loss(self,T_now=200):
        '''
        ## Get the derivation of the total energy of starbodies at specified time
        '''
        if T_now>self.Coalescence_time:
            print('---Coalescence Time {}s Reached!, Current Time: {}s---'.format(self.Coalescence_time,T_now))
            return 0
        Reduced_mass=Reduce_mass(self.M[0],self.M[1])
        Distance=self.Distance(T_now)
        Orbit_Spin=Orbit_spin(self.M[0],self.M[1],Distance)
        return N*(BC.G*Reduced_mass**2*Distance**4*Orbit_Spin**6)/(BC.c**5)
    def Detect_Signal(self,Obser_R=[1e3,1e3,1e3],T_now=200):
        '''
        ## Get the interferometer strain signal of LIGO\n
        ### Parameters:
        - Obser_R: the position vector of the observer
        - T_now: the time at the place of observation
        
        ### Return:
            The Detected Gravitational Wave Field Signal(scalar)
        '''
        if T_now>self.Coalescence_time:
            print('---Coalescence Time {}s Reached!, Current Time: {}s---'.format(self.Coalescence_time,T_now))
            return 0
        coef=Reduce_mass(self.M[0],self.M[1])/(8*np.sum(self.M))
        r=self.Distance(T_now)
        spin=Orbit_spin(self.M[0],self.M[1],r)
        coef*=Schwarz_Radius(np.sum(self.M))**2/(r*BMech.Vec_Length(Obser_R))
        theta=BMech.Vec_Angle(Obser_R,[0,0,1])
        # beta=np.arcsin(np.cos(theta)*np.cos(spin*T_now))
        beta=np.arcsin(np.cos(theta))
        coef*=np.sin(beta)*np.cos(beta)*np.sin(2*theta)*np.cos(2*spin*T_now)
        return coef

    def Orbit_Phase(self,T_now=200):
        '''
        ## Get the r1/r2 phase at the specified time, phase=0 when r1 along the x+ axis
        '''
        if T_now<self.Coalescence_time:
            tsc=2*BC.G*np.sum(self.M)*math.pow(Reduce_mass(self.M[0],self.M[1])/np.sum(self.M),3/5)/BC.c**3
            phase=math.pow(0.4,5/8)*(math.pow(self.Coalescence_time/tsc,5/8)-math.pow((self.Coalescence_time-T_now)/tsc,5/8))
            return phase
        else:
            print('---Coalescence Time {}s Reached!, Current Time: {}s---'.format(self.Coalescence_time,T_now))
            return 0
    def DATA(self,Dt=0.1,T_init=0,T_end=10,Obser_R=[1e3,1e3,1e3],imgoutput='',Plot=True):
        '''
        ## Plot data of Gravitational wave
        ### Parameters:
        - Dt: the time interval between nearby points
        - T_init: the start time of data
        - T_end: the end time of data
        - Obser_R: the position vector of the observer
        - imgoutput: the folder to save the ploted image, if it's empty, no image will be saved
        - Plot: plot the data curve
        '''
        # Prepare the data
        dis=[]
        grav=[]
        signal=[]
        phase=[]
        if T_end>self.Coalescence_time:
            print('---Coalescence Time {}s Reached!, Current End Time: {}s---'.format(self.Coalescence_time,T_end))
            return 0
        if T_init>=T_end:
            print('Start time {}s is larger than End time {}s!'.format(T_init,T_end))
            return 0

        # Calculate
        time=range(1+int((T_end-T_init)/Dt))
        bar=tqdm(time,mininterval=1)
        bar.set_description('Calculating Gravitational Wave Data')
        for i in bar:
            dis.append(self.Distance(T_now=T_init+i*Dt))
            grav.append(BMech.Vec_Length(self.Grav_Wave(Obser_R=Obser_R,T_now=T_init+i*Dt)))
            signal.append(self.Detect_Signal(Obser_R=Obser_R,T_now=T_init+i*Dt))
            phase.append(np.sin(self.Orbit_Phase(T_now=T_init+i*Dt)))
        time=np.array(time)
        time=time*Dt+T_init
        dis=np.array(dis)
        grav=np.array(grav)
        signal=np.array(signal)
        phase=np.array(phase)
        
        # Plot data
        plt.figure(figsize=(30,20))
        plt.subplot(221)
        plt.plot(time,dis)
        plt.title('Distance between the binary planets')
        plt.xlabel('Time/s')
        plt.ylabel('Distance/m')
        
        plt.subplot(222)
        plt.plot(time,grav)
        plt.title('Strength of the gravitatioanl wave')
        plt.xlabel('Time/s')
        plt.ylabel('Gravitatioanl wave strength')

        plt.subplot(223)
        plt.plot(time,signal)
        plt.title('Signal strength detected by laser interferomater')
        plt.xlabel('Time/s')
        plt.ylabel('Signal Strength')

        plt.subplot(224)
        plt.plot(time,phase)
        plt.title('Orbit pahse of the binary planets')
        plt.xlabel('Time/s')
        plt.ylabel('Phase/sin()')
        
        # save the data fig
        if imgoutput!='':
            try:
                os.makedirs(imgoutput)
            except:
                pass
            path=os.path.join(imgoutput,'{}_Ti_{}_Te_{}_Dt_{}_Obser_{}.jpg'.format(self.name,T_init,T_end,Dt,Obser_R))
            plt.savefig(path)
            print('---Data image saved as {}---'.format(path))
        # Show the image
        if Plot==True:
            plt.show()# DONOT put plt.show() in front of plt.savefig!!!
        return dis,grav,signal,phase
    def __SaveData(self,Dt=0.1,T_init=0,T_end=10,Obser_R=[1e3,1e3,1e3],output=''):
        # # save the data
        # if output!='':
        #     try:
        #         os.makedirs(output)
        #     except:
        #         pass
        #     path=os.path.join(output,'{}_Ti_{}_Te_{}_Dt_{}_Obser_{}.pkl'.format(self.name,T_init,T_end,Dt,Obser_R))
        #     with open(path,'wb')as f:
        #         pkl.dump('data',f)
        #     print('---Data saved as {}---'.format(path))
        pass

class CGW(SB.StarBody):
    '''
    # Continuous gravitational waves
    Continuous gravitational waves are thought to be produced by a single spinning massive object like a neutron star. 
    Any bumps on or imperfections in the spherical shape of this star will generate gravitational waves as it spins. 
    If the spin-rate of the star stays constant, so too are the gravitational waves it emits. 
    That is, the gravitational wave is continuously the same frequency and amplitude (like a singer holding a single note). 
    That's why these are called “Continuous Gravitational Waves”.
    ### Parameters:
    - Mass: the mass of the pulsar
    - Radius: the radius of the pulsar
    - Spin: the saclar spin speed of the planet, in rad/s, the spin will be automatically aligned to z-axis
    - Name: the name of the pulsar
    - Meg: the megnatic dipole of the pulsar
    - RotI: the rotation inertia of the pulsar

    All parameters meet SI standard.
    '''
    def __init__(self,Name='PulsarAX01',Mass=1e5*BC.M_sun,Radius=1e-3*BC.R_earth,Spin=1,Meg=[1e3,1e3,1e3],RotI=[1.1,0.9,1.2]):
        super().__init__(Position=[0,0,0],Velocity=[0,0,0],Mass=Mass,Radius=Radius,Spin=[0,0,Spin],Name=Name)
        if BMech.Vec_Length(self.spin)>self.ad_prop['Max_spin']:
            self.spin=self.ad_prop['Max_spin']
            print('---Spin Velocity Boomed, Refreshed as the Maximum Value---')
        self.MegDio=np.array(Meg)
        self.MegSur=BC.mu0*self.MegDio/(2*math.pi*self.Radius**3)
        self.RotI=2*self.mass*self.Radius**2/5*np.array(RotI)
    def __str__(self):
        str1='---Basic Information of the Fast Spinning Starbody {}---\n'.format(self.name)
        str2=PT.PrettyTable()
        str2.field_names=['Properties','Value']
        str2.add_rows([['Mass(kg)',self.mass],
                        ['Density(kg/m3)',self.rho],
                        ['Radius(m)',self.Radius],
                        ['Spin Velocity(rad/s)',self.spin],
                        ['Magnetic Dipole(A.m2)',self.MegDio],
                        ['Surface Megnatic Field(T)',self.MegSur],
                        ['The moment of inertia(kg.m2)',self.RotI]])
        print(str1)
        print(str2)
        str3='\n---Advanced Properties of the StarBody {}---\n'.format(self.name)
        str4=PT.PrettyTable()
        str4.field_names=['Properties','Value']
        str4.add_rows([['1st order velocity(m/s)',self.ad_prop['V1']],
                        ['2nd order velocity(m/s)',self.ad_prop['V2']],
                        ['Black Hole',self.ad_prop['Black_hole']],
                        ['Vision Radius(m)',self.ad_prop['Vision_radius']],
                        ['Maximum Spin Velocity(rad/s)',self.ad_prop['Max_spin']]])
        print(str3)
        print(str4)
        # str5=('\n---Sphere Model Properties of the StarBody {}---\n'.format(self.name))
        # str6=PT.PrettyTable()
        # str6.field_names=['Properties','Value']
        # str6.add_rows([['Sphere Model Data shape',self.Sphere.shape],
        #                 ['2nd-Index of the Sphere Center Plane',(self.Sphere.shape[1]-1)/2],
        #                 ['Segmentation D_Angle(degree)',self.DAngle],
        #                 ['Outer Sphere Radius(m)',self.Radius]])
        # print(str5)
        # print(str6)
        str7='---More are available because of your exploration! And all properties meet SI standard---\n'
        
        return str7
    def __DMegEnergy_Loss(self,T_now=200):
        '''
        ## Get the derivation of the total energy of starbody at specified time, due to megnatic field
        '''
        Meg_t=np.dot(np.array([1,1,0]),self.MegDio)
        return -(BC.mu0*Meg_t**2*self.spin[3]**4)/(6*math.pi*BC.c**3)
    def __MegSpin(self,T_now=100):
        '''
        ## Get the Spin velocity in rad at time T_now, evolve with Megnatic field
        '''
        spin=np.sqrt(1/(T_now*(BC.mu0*self.Radius**6*np.dot(np.array([1,1,0]),self.MegSur)**2)/(3*math.pi*BC.c**3*self.RotI[2])+1/self.spin[2]**2))
        spin=np.array([0,0,spin])
        return spin
    def GWSpin(self,T_now=100):
        '''
        ## Get the Spin velocity in rad at time T_now, evolve with Gravitational Wave field
        '''
        spin=T_now*6*N*BC.G*self.RotI[2]/BC.c**5+1/self.spin[2]**6
        spin=math.pow(1/spin,1/6)
        spin=np.array([0,0,spin])
        return spin
    def __DMegSpin(self,T_now=100):
        return -self.MegSpin(T_now=T_now)[2]**3*(BC.mu0*self.Radius**6*np.dot(np.array([1,1,0]),self.MegSur)**2)/(6*math.pi*BC.c**3*self.RotI[2])
    def __DGWSpin(self,T_now):
        return -N*BC.G*self.RotI[2]/BC.c**5*self.GWSpin(T_now=T_now)[2]**5
    def __DGWEnergy_Loss(self,T_now=200):
        '''
        ## Get the derivation of the total energy of starbody at specified time, due to Gravitational Wave field
        '''
        return -N*BC.G*(self.RotI[0]-self.RotI[1])**2*self.GWSpin(T_now=T_now)**6/BC.c**5
    def __OverAllAmpGW(self,T_now=100,Obser_R=[1e3,1e3,1e3]):
        return 16*math.pi*BC.G*np.abs(self.RotI[0]-self.RotI[1])*(self.GWSpin(T_now=T_now)/(2*math.pi))**2/(BC.c**4*BMech.Vec_Length(Obser_R))

    def Detect_Signal(self,Obser_R=[1e3,1e3,1e3],T_now=200):
        '''
        ## Get the interferometer strain signal of LIGO\n
        ### Parameters:
        - Obser_R: the position vector of the observer
        - T_now: the time at the place of observation
        
        ### Return:
            The Detected Polarized Gravitational Wave Field, [h+,hx]
        '''
        i=BMech.Vec_Angle(Obser_R,self.spin)
        w=self.GWSpin(T_now=T_now)[2]
        w0=self.spin[2]
        h_plus=(-2/BMech.Vec_Length(Obser_R))*(1+np.cos(np.deg2rad(i))**2)*(self.RotI[1]-self.RotI[0])*(w-w0)**2*np.cos(2*(w-w0)*T_now)
        h_cross=(-4/BMech.Vec_Length(Obser_R))*np.cos(np.deg2rad(i))*(self.RotI[1]-self.RotI[0])*(w-w0)**2*np.sin(2*(w-w0)*T_now)
        return [h_plus,h_cross]
    def DATA(self,Dt=10,T_init=0,T_end=1e5,Obser_R=[1e3,1e3,1e3],imgoutput='',Plot=True):
        '''
        ## Plot data of Gravitational wave
        ### Parameters:
        - Dt: the time interval between nearby points
        - T_init: the start time of data
        - T_end: the end time of data
        - Obser_R: the position vector of the observer
        '''
        # Prepare the data
        spin=[]
        signal=[]
        if T_init>=T_end:
            print('Start time {}s is larger than End time {}s!'.format(T_init,T_end))
            return 0

        # Calculate
        time=range(1+int((T_end-T_init)/Dt))
        bar=tqdm(time,mininterval=1)
        bar.set_description('Calculating Gravitational Wave Data')
        for i in bar:
            spin.append(self.GWSpin(T_now=T_init+i*Dt)[2])
            signal.append(self.Detect_Signal(Obser_R=Obser_R,T_now=T_init+i*Dt))

        time=np.array(time)
        time=time*Dt+T_init
        spin=np.array(spin)
        signal=np.array(signal)
        
        # Plot data
        plt.figure(figsize=(16,4))
        plt.subplot(121)
        plt.plot(spin)
        plt.title('Spin Velocity of Pulsar')
        plt.xlabel('Time/s')
        plt.ylabel('rad/s')
        
        plt.subplot(122)
        plt.plot(signal)
        plt.title('Strength of the Polarized Signal h+/hx')
        plt.xlabel('Time/s')
        plt.ylabel('Signal Strength')
        
        # save the data fig
        if imgoutput!='':
            try:
                os.makedirs(imgoutput)
            except:
                pass
            path=os.path.join(imgoutput,'{}_Ti_{}_Te_{}_Dt_{}_Obser_{}.jpg'.format(self.name,T_init,T_end,Dt,Obser_R))
            plt.savefig(path)
            print('---Data image saved as {}---'.format(path))
        # Show the image
        if Plot==True:
            plt.show()# DONOT put plt.show() in front of plt.savefig!!!
        return spin,signal
    

def Reduce_mass(m1,m2):
    return m1*m2/(m1+m2)

def Total_E(m1,m2,Distance):
    return -BC.G*m1*m2/(2*Distance)

def Orbit_spin(m1,m2,Distance):
    return BC.G*(m1+m2)/Distance**3

def Schwarz_Radius(m):
    return 2*BC.G*m/BC.c**2





# Always prefer setuptools over distutils
from prometheus_client import Summary
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")


setup(
    name="PurpleMicroStar",
    version="1.0.3",
    license='GPL-3.0',
    description='Universe is waiting for us to explore! ',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://airscker.github.io/Purple-Micro-Star/",
    author="Airscker/Yufeng Wang",
    author_email="airscker@gmail.com",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",        
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
    ],
    keywords="Multibody Motion, Gravitational Wave, Science",  # Optional
    # When your source code is in a subdirectory under the project root, e.g.
    # `src/`, it is necessary to specify the `package_dir` argument.
    # package_dir={"": "src"},  # Optional
    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(),
    python_requires=">=3.5, <4",
    install_requires=["numpy","click","tqdm","prettytable","matplotlib"],  # Optional
    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ pip install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    # extras_require={  # Optional
    #     "dev": ["check-manifest"],
    #     "test": ["coverage"],
    # },
    # If there are data files included in your packages that need to be
    # installed, specify them here.
    package_data={  # Optional
        "sample": ["Multibody-3.csv","Multibody-6.csv"],
    },
    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # data_files=[("my_data", ["data/data_file"])],  # Optional
    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    # entry_points={  # Optional
    #     "console_scripts": [
    #         "sample=sample:main",
    #     ],
    # },
    # List additional URLs that are relevant to your project as a dict.
    #
    # This field corresponds to the "Project-URL" metadata fields:
    # https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use
    #
    # Examples listed include a pattern for specifying where the package tracks
    # issues, where the source is hosted, where to say thanks to the package
    # maintainers, and where to support the project financially. The key is
    # what's used to render the link text on PyPI.
    # project_urls={  # Optional
    #     "Home page": "https://airscker.github.io/Purple-Micro-Star/"
    # },
)



import numpy as np
import PurpleMicroStar.BasicConst as BC
'''
### This module contains some basic mathematical and physical algorithms

@AUTHOR: Yufeng Wang
@DATE: 2022-6
@LICENSE: GPL-V3 license

CopyRight (C) Yufeng-Wang/Airscker, 2022-6

More information about this module please see: https://airscker.github.io/Purple-Micro-Star/
'''

def G_Force(m1=1,m2=1,pos1=[0,0,0],pos2=[1,1,1]):
    '''
    ## Get two planets' gravity force
    ### Parameters:
    - m1,m2: the mass of two planets
    - pos1,pos2: the PCI coordinate position of two planets, in the form of list [x,y,z]
    ### Return:
    - The gravity force of two planet, in the form of nparray [fx,fy,fz]
    - If two planets too close to get result, None is returned
    '''
    pos1=np.array(pos1)
    pos2=np.array(pos2)
    rel_pos=np.sqrt(np.sum((pos2-pos1)**2))
    if rel_pos!=0:
        output=(pos2-pos1)*BC.G*m1*m2/rel_pos**3
    else:
        print('Position corrupted!')
        return None
    return output

def Vec_Project(vec1=[1,0,0],vec2=[1,1,1]):
    '''
    ## Project the 2nd vector to 1st vector
    ### Parameters:
    - vec1: the vector selected as base, in the form of list [x,y,z]
    - vec2: the vector need to be projected, in the form of list [x,y,z]
    ### Return:
        the projected vector, in the form of nparray [x,y,z]
    '''
    vec1=np.array(vec1)
    vec2=np.array(vec2)
    output=np.dot(vec1,vec2)*vec1/np.sum(vec1**2)
    return output

def Vec_Angle(vec1=[1,0,0],vec2=[1,1,1]):
    '''
    ## Get two vectors' angle
    ### Parameters:
        vec1,vec2: Vectors, in the form of list [x,y,z]
    ### Return:
        The angle of two vectors, in degree
    '''
    vec1=np.array(vec1)
    vec2=np.array(vec2)
    output=np.rad2deg(np.arccos(np.dot(vec1,vec2)/(Vec_Length(vec1)*Vec_Length(vec2))))
    return output

def AngleXYZ(vec,plane='XY'):
    '''
    ## Get the angle between the vector and specified plane
    ### Parameters:
        plane: XY,YZ,XZ available
    ### Return:
        The angle in degree
    '''
    vec_c=vec.copy()
    plus=1
    if plane=='XY':
        vec_c[0]=0
        if vec[2]<0:
            plus=-1
        return plus*(90-Vec_Angle(vec_c,[0,0,1]))
    if plane=='XZ':
        vec_c[0]=0
        if vec[1]<0:
            plus=-1
        return plus*(90-Vec_Angle(vec_c,[0,1,0]))
    if plane=='YZ':
        vec_c[0]=0
        if vec[0]<0:
            plus=-1
        return plus*(90-Vec_Angle(vec_c,[1,0,0]))

def Vec_Length(vec=[1,1,1]):
    '''
    ## Get the length of vector
    '''
    vec=np.array(vec)
    return np.sqrt(np.sum(vec**2))

def Vec_Norm(vec=[10,6,8]):
    '''
    ## Get normalized vector
    '''
    return vec/Vec_Length(vec)

def Rotate_XYZ(vec=[1,1,-1],rad=0.1,Axis='X'):
    '''
    ## Rotate vector clockwise
    '''
    if Axis=='X':
        Rotate=np.array([[1,0,0],[0,np.cos(rad),-np.sin(rad)],[0,np.sin(rad),np.cos(rad)]])
    if Axis=='Y':
        Rotate=np.array([[np.cos(rad),0,np.sin(rad)],[0,1,0],[-np.sin(rad),0,np.cos(rad)]])
    if Axis=='Z':
        Rotate=np.array([[np.cos(rad),-np.sin(rad),0],[np.sin(rad),np.cos(rad),0],[0,0,1]])

    return np.dot(Rotate,np.array(vec))



import PurpleMicroStar.BasicConst as BC
import PurpleMicroStar.BasicMech as BMech
import matplotlib.pyplot as plt
import numpy as np
import pickle as pkl
import math
import os
import prettytable as PT
from tqdm import tqdm

'''
### This module is built for simulating single celestial body, as well as calculating basic properties

@AUTHOR: Yufeng Wang
@DATE: 2022-6
@LICENSE: GPL-V3 license

CopyRight (C) Yufeng-Wang/Airscker, 2022-6

More information about this module please see: https://airscker.github.io/Purple-Micro-Star/
'''

class StarBody():
    '''
    ### Parameters:
    - Position: the position of the planet, in PCI coordinate with the form [x,y,z]
    - Velocity: the velocity of the planet, in PCI coordinate with the form [vx,vy,vz]
    - Mass: the mass of the planet
    - Radius: the radius of the planet
    - Spin: the spin speed of the planet, in rad/s
    - Name: the name of the planet

    All parameters meet SI standard.
    '''
    
    def __init__(self,
                Position=[1,1,1],
                Velocity=[10000,10000,10000],
                Mass=5.965e24,
                Radius=6360e3,
                Spin=[0,0,7.292e-5],
                Name='Earth',
                manybody=[]):
        # Basic properties
        self.pos=np.array(Position)
        self.vel=np.array(Velocity)
        self.mass=Mass
        self.Radius=Radius
        self.spin=np.array(Spin)
        self.name=Name
        self.rho=3*self.mass/(4*math.pi*self.Radius**3)
        self.force=np.array([0.0,0.0,0.0])
        # self.Sphere=self.SphereModel()
        self.Sphere=None
        self.iter=0
        self.Ctime=0
        # Advanced properties
        self.ad_prop={
            'V1':math.sqrt(BC.G*self.mass/self.Radius),
            'V2':math.sqrt(2*BC.G*self.mass/self.Radius),
            'Roche_limit':0,
            'Black_hole':bool(self.Radius<=BC.G*self.mass/BC.c**2),
            'Vision_radius':BC.G*self.mass/BC.c**2,
            'Max_spin':math.sqrt(BC.G*self.mass/(self.Radius**3))
        }
        if manybody!=[]:
            self.Force(manybody)
            self.Roche_limit(manybody)
        # Other parameters
        self.DAngle=1
    def __str__(self):
        str1='---Basic Information of the StarBody {}---\n'.format(self.name)
        str2=PT.PrettyTable()
        str2.field_names=['Properties','Value']
        str2.add_rows([['Position(m)',self.pos],
                        ['Velocity(m/s)',self.vel],
                        ['Mass(kg)',self.mass],
                        ['Density(kg/m3)',self.rho],
                        ['Radius(m)',self.Radius],
                        ['Spin Velocity(rad/s)',self.spin],
                        ['Gravity Force(N)',self.force],
                        ['Time Since Launch(s)',self.Ctime],
                        ['Iteration Times',self.iter]])
        print(str1)
        print(str2)
        str3='\n---Advanced Properties of the StarBody {}---\n'.format(self.name)
        str4=PT.PrettyTable()
        str4.field_names=['Properties','Value']
        str4.add_rows([['1st order velocity(m/s)',self.ad_prop['V1']],
                        ['2nd order velocity(m/s)',self.ad_prop['V2']],
                        ['Roche limit proportion',self.ad_prop['Roche_limit']/self.mass],
                        ['Black Hole',self.ad_prop['Black_hole']],
                        ['Vision Radius(m)',self.ad_prop['Vision_radius']],
                        ['Maximum Spin Velocity(rad/s)',self.ad_prop['Max_spin']]])
        print(str3)
        print(str4)
        # str5=('\n---Sphere Model Properties of the StarBody {}---\n'.format(self.name))
        # str6=PT.PrettyTable()
        # str6.field_names=['Properties','Value']
        # str6.add_rows([['Sphere Model Data shape',self.Sphere.shape],
        #                 ['2nd-Index of the Sphere Center Plane',(self.Sphere.shape[1]-1)/2],
        #                 ['Segmentation D_Angle(degree)',self.DAngle],
        #                 ['Outer Sphere Radius(m)',self.Radius]])
        # print(str5)
        # print(str6)
        if self.ad_prop['Roche_limit']/self.mass>=1:
            print('\n---Starbody {} Crashed, RocheLimit {}(kg) Reached---\n'.format(self.name,self.ad_prop['Roche_limit']))
        str7='---More are available because of your exploration! And all properties meet SI standard---\n'
        
        return str7
    def Force(self,manybody=[]):
        '''
        ## Get the gravity force of this planet(Multibody gravity force)
        ### Parameter:
            manybody: the list of other existing planet bodies' Object of StarBody
        ### Return:
            The Multibody gravity force of this planet
        '''
        try:
            for i in range(len(manybody)):
                self.force+=BMech.G_Force(self.mass,manybody[i].mass,self.pos,manybody[i].pos)
        except:
            print('Force Calculation Error!')
            return None
        return self.force
    def Iteration(self,dt=0.001,iterCount=1):
        '''
        ## Get the motion information of this planet at specified time
        ### Parameters:
        - dt: the time period per motion used
        - iterCount: the iteration times of dt
        '''
        if self.ad_prop['Roche_limit']/self.mass<1:
            self.iter+=iterCount
            self.Ctime+=iterCount*dt
            try:
                for i in range(iterCount):
                    self.pos=self.vel*dt+self.pos+self.force*dt**2/(2*self.mass)
                    self.vel=self.vel+self.force*dt/self.mass
            except:
                print('Iteration error!')
                return None
            return 0
        else:
            return 0
    def Roche_limit(self,manybody=[]):
        '''
        ## Get all starbodies' roche limit proportion
        '''
        sum=0
        for i in range(len(manybody)):
            sum+=(manybody[i].mass*2*self.Radius**3)/(math.sqrt(np.sum(self.pos-manybody[i].pos)**2))**3
        self.ad_prop['Roche_limit']=sum
        return sum
    def __SphereModel(self,DAngle=0.1):
        '''
        ### Parameters:
        - DAngle: the angle between nearby points
        - DR: the distance between nearby spheres
        - output: the folder to save the sphere data file

        ### Return:
        - Normalized Sphere Model\n
            Shape of data: (2,NumberOfPlanes,DividedPatten)
            -- 2: X/Y axis
            -- NumberOfPlanes: Number of Image Planes (DAngle=1 -> 90*2+1)
            -- DividedPatten: 360/DAngle
            
        This function gives us with a outer sphere model, when we want to build whole ball model we just need to multiply 1/proportion to get the inner sphere model
        '''
        X=[]
        Y=[]
        self.DAngle=DAngle
        bar=tqdm(range(int(90.0/DAngle+1)),mininterval=1)
        bar.set_description('Building Sphere Star Model of {}'.format(self.name))
        for i in bar:
            x=self.Radius*np.cos(np.deg2rad(i*DAngle))*np.cos(np.deg2rad(np.arange(0,360,DAngle)))
            y=self.Radius*np.cos(np.deg2rad(i*DAngle))*np.sin(np.deg2rad(np.arange(0,360,DAngle)))
            X.append(x)
            Y.append(y)
            if i!=0:
                X.insert(0,x)
                Y.insert(0,-y)
        self.Sphere=np.array([X,Y])
        return self.Sphere
    def __SaveSphereModel(self,output=''):
        if output!='':
            try:
                os.makedirs(output)
            except:
                pass
            print('---Saving StarBody Sphere Model---')
            path='{}\\SphereModel_R_{}_DAngle_{}.pkl'.format(output,self.Radius,self.DAngle)
            file=open(path,'wb')
            pkl.dump(self.Sphere,file)
            print('---Sphere Model Data Saved as {}---'.format(path))
        return path






'''
# Some Basic Constants in Physics
## Avaiable Constants:\n
    Gravitational Constant: G=6.67259e-11\n
    Speed of light: c=299792458\n    
    Light year: Ly=9.46e12\n
    Astronomical Unit: AU=1.496e12\n
    Mass of planets:\n
        M_sun=1.989e30\n
        M_earth=5.974e24\n    
    Plank Constant: Plank_h=6.6262e-34\n
    Charge of electron: Qe=1.6022e-19\n
    Mass of basic particles:\n
        Me=9.1095e-31\n
        Mp=1.6726e-27\n
        Mn=1.67493e-27\n
    Avogadro Constant: Na=6.022e26\n
    Steven-Boltzman Constant: Sigma=5.67e-8\n    
    Electric Vault: eV=1.6022e-19\n    
    Radius of planets:\n
        R_earth=6371393\n
        R_sun=696300000\n    
    Spin of planets:\n
        S_earth=[0,0,7.292e-5]\n
    permeability:\n
        mu0=4*math.pi*1e-7

@AUTHOR: Yufeng Wang
@DATE: 2022-6
@LICENSE: GPL-V3 license

CopyRight (C) Yufeng-Wang/Airscker, 2022-6

More information about this module please see: https://airscker.github.io/Purple-Micro-Star/
'''

import math
# Gravity
G=6.67259e-11
# Speed of light
c=299792458
# Light year
Ly=9.46e12
# Celestial distance
AU=1.496e12
# Mass of planets
M_sun=1.989e30
M_earth=5.974e24
# Plank const
Plank_h=6.6262e-34
# Q of electron
Qe=1.6022e-19
# Mass of basic particles
Me=9.1095e-31
Mp=1.6726e-27
Mn=1.67493e-27
# Avogadro constant
Na=6.022e26
# Steven-Boltzman const
Sigma=5.67e-8
# Electric Vault
eV=1.6022e-19
# Radius of planet
R_earth=6371393
R_sun=696300000
# Spin of planet
S_earth=[0,0,7.292e-5]
# Megnatic
mu0=4*math.pi*1e-7
